/*
 * Copyright (C) 2005-2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file db.h data structures common to all database tools */

int find_audio_files(const char *dirname, int (*f)(const char *, const char *));

/**
 * describes one of para_server's supported database tools
 *
 * There is exactly one such struct for each supported database tool.  During
 * the startup part of para_server the \a init() function of the activated
 * database tool gets called which fills in the other members.

 *
 *
 */
struct dbtool {
/**
 * name name of this database tool
 */
const char *name;
/**
 * the database init routine
 *
 * It should check its command line options and do all necessary initialization
 * like connecting to a database server.
 *
 * A negative return value indicates an initialization error and means that
 * this database tool should be ignored for now (it may later be activated
 * again via the cdt command).
 *
 * If \a init() returns success (non-negative return value), it must have
 * initialized in all non-optional function pointers of the given dbtool
 * struct. Moreover, \a cmd_list must point to a NULL-terminated array which
 * holds the list of all commands that are supported by this database tool.
 */
int (*init)(struct dbtool *self);
/**
 * list of commands supported by this dbtool
 */
struct server_command *cmd_list;
/**
 * pointer to function returning list of at most \a num audio files to be
 * streamed next
 *
 * \a get_audio_file_list() must return a pointer to a array of at most \a num
 * char* pointers (terminated by a NULL pointer), or NULL on errors.  Both the
 * array and its contents must be dynamically allocated and are freed by the
 * caller.
 *
*/
char **(*get_audio_file_list)(unsigned int num);
/**
 *
 * the update hook
 *
 * The \a update_audio_file pointer is optional and need not be supplied. In this
 * case it is not neccessary to init this pointer from within init(). If
 * \a update_audio_file is non-NULL, the function it points to gets called
 * whenever a new audio file was successfully loaded and is going to be
 * streamed by any of paraslash's senders. The full path of the audio file is
 * passed \a update_audio_file().
 *
 */
void  (*update_audio_file)(char *audio_file);
/**
 *
 * shutdown this database tool and free all resources
 *
 * This gets called whenever the database tool changes (via the cdt command),
 * or when para_server receives the HUP signal, or when para_server shuts down.
 * It is assumed to succeed.
*/
void (*shutdown)(void);
};

int mysql_dbtool_init(struct dbtool*);
int dopey_dbtool_init(struct dbtool*);

